import { Watch, State, Prop, Component, Host, h } from '@stencil/core';
import { Client } from '../../utils/client';
/**
 * @slot (default) - The markup for anything you want to test. Maybe a component or a pattern.
 * @part image - The image returned from Figma, if available.
 */
export class FtlHolster {
  constructor() {
    /**
     * A unique name for your test to display in the UI.
     */
    this.name = 'Test Case';
    this.opacity = '1';
    this.view = 'toggle';
    this.mode = 'side';
    this.activeToggle = 'figma';
    this.showGrid = true;
    this.loading = true;
    this.error = 'hi';
    this.status = 'idle';
    this.camelize = s => s.replace(/-./g, x => x[1].toUpperCase());
  }
  handleParamChanges() {
    this._fetchImage();
  }
  componentWillLoad() {
    this._fetchImage();
  }
  _fetchImage() {
    if (this.accessToken && this.fileId) {
      this.status = 'loading';
      const client = new Client(this.accessToken, this.fileId);
      client.getNode(this.node).then((node) => {
        this.imageUrl = node;
        this.status = 'success';
      }).catch(e => {
        console.log(e.message);
        this.status = 'fail';
      });
    }
  }
  updateActiveToggle(e) {
    this.activeToggle = e.target.value;
  }
  get figmaLoadingBlock() {
    return (h("div", null, "Your content is loading"));
  }
  get loadingSvg() {
    return (h("svg", { width: "55", height: "80", viewBox: "0 0 55 80", xmlns: "http://www.w3.org/2000/svg", fill: "#FFF" },
      h("g", { transform: "matrix(1 0 0 -1 0 80)" },
        h("rect", { width: "10", height: "20", rx: "3" },
          h("animate", { attributeName: "height", begin: "0s", dur: "4.3s", values: "20;45;57;80;64;32;66;45;64;23;66;13;64;56;34;34;2;23;76;79;20", calcMode: "linear", repeatCount: "indefinite" })),
        h("rect", { x: "15", width: "10", height: "80", rx: "3" },
          h("animate", { attributeName: "height", begin: "0s", dur: "2s", values: "80;55;33;5;75;23;73;33;12;14;60;80", calcMode: "linear", repeatCount: "indefinite" })),
        h("rect", { x: "30", width: "10", height: "50", rx: "3" },
          h("animate", { attributeName: "height", begin: "0s", dur: "1.4s", values: "50;34;78;23;56;23;34;76;80;54;21;50", calcMode: "linear", repeatCount: "indefinite" })),
        h("rect", { x: "45", width: "10", height: "30", rx: "3" },
          h("animate", { attributeName: "height", begin: "0s", dur: "2s", values: "30;45;13;80;56;72;45;76;34;23;67;30", calcMode: "linear", repeatCount: "indefinite" })))));
  }
  /**
   * Prevents issue where Figma returns an image that is longer than the coded component in Overlay mode.
   * Because the figma image is positioned absolute and I have no idea how to do it with CSS.
   */
  setWidth() {
    const width = this.imgElement.width;
    const height = this.imgElement.height;
    if (width && height) {
      this.gridElement.setAttribute('style', `min-width: ${width}px; min-height: ${height + 50}px;`);
    }
  }
  updateView(attribute, e) {
    const formatAttr = this.camelize(attribute);
    this[formatAttr] = e;
  }
  render() {
    return (h(Host, null,
      h("div", { class: "ftl-holster" },
        h("header", null,
          h("h2", null, this.name)),
        h("div", { class: {
            "container": true,
            "container__overlay": this.view === 'overlay',
            "container__toggle": this.view === 'toggle',
            "activeToggle--figma": this.activeToggle === 'figma',
            "activeToggle--code": this.activeToggle === 'code'
          } },
          h("div", { ref: el => this.gridElement = el, class: { "grid": true, "grid--overlay": this.showGrid } },
            h("div", { class: "figma__container" },
              h("div", { class: "image__label" }, "Figma"),
              this.status === 'loading' ? (this.loadingSvg) : null,
              this.status === 'fail' ? (h("div", null, " it failed")) : null,
              this.status === 'success' ? (h("img", { part: "image", onLoad: () => this.setWidth(), ref: el => this.imgElement = el, src: this.imageUrl, style: {
                  opacity: this.view === 'overlay' ? `${this.opacity}` : '1'
                }, alt: "" })) : null),
            h("div", { class: "code__container" },
              h("div", { class: "image__label" }, this.view === 'overlay' ? "Figma and Code" : "Code"),
              h("slot", null))),
          h("ftl-buckle", { "active-toggle": this.activeToggle, "show-grid": this.showGrid, opacity: this.opacity, view: this.view, "onFtl-on-change": (e) => this.updateView(e.detail.attribute, e.detail.value) })))));
  }
  static get is() { return "ftl-holster"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["ftl-holster.css"]
  }; }
  static get styleUrls() { return {
    "$": ["ftl-holster.css"]
  }; }
  static get properties() { return {
    "accessToken": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Your Figma Personal Access Token. Find it under account settings from the top-left menu inside Figma."
      },
      "attribute": "access-token",
      "reflect": false
    },
    "fileId": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Your Figma file ID. https://www.figma.com/file/MYFILEID/my-file"
      },
      "attribute": "file-id",
      "reflect": false
    },
    "node": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The node ID of the component you want to preview."
      },
      "attribute": "node",
      "reflect": false
    },
    "name": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "A unique name for your test to display in the UI."
      },
      "attribute": "name",
      "reflect": false,
      "defaultValue": "'Test Case'"
    },
    "opacity": {
      "type": "string",
      "mutable": true,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "opacity",
      "reflect": true,
      "defaultValue": "'1'"
    },
    "view": {
      "type": "string",
      "mutable": true,
      "complexType": {
        "original": "viewOptions",
        "resolved": "\"overlay\" | \"side-by-side\" | \"toggle\"",
        "references": {
          "viewOptions": {
            "location": "import",
            "path": "../ftl-buckle/ftl-buckle"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "view",
      "reflect": true,
      "defaultValue": "'toggle'"
    },
    "activeToggle": {
      "type": "string",
      "mutable": true,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "active-toggle",
      "reflect": true,
      "defaultValue": "'figma'"
    },
    "showGrid": {
      "type": "boolean",
      "mutable": true,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "show-grid",
      "reflect": true,
      "defaultValue": "true"
    }
  }; }
  static get states() { return {
    "imageUrl": {},
    "mode": {},
    "loading": {},
    "error": {},
    "status": {}
  }; }
  static get watchers() { return [{
      "propName": "accessToken",
      "methodName": "handleParamChanges"
    }, {
      "propName": "fileId",
      "methodName": "handleParamChanges"
    }]; }
}
